#include "config.h"
#include <stdio.h>
#include <sys/types.h>
#include "common.h"
#ifdef X68
#include "tty_x68.h"
#else
#ifdef WIN32
#include "tty_w32.h"
#else
#ifdef OS2
#include "tty_os2.h"
#else
#ifdef DOS
#include "tty_dos.h"
#else
#include "tty.h"
#include <termios.h>
#endif /* DOS */
#endif /* OS2 */
#endif /* WIN32 */
#endif /* X68K */
#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if 0
# define dprintf(x)	fprintf x
#else
# define dprintf(x)
#endif

static int	IRfd = -1;
extern int verbose;

void
IRsetfd(fd)
     int	fd;
{
  dprintf((stderr, "IRfd = %x\n", fd));
  IRfd = fd;
}

int
IRgetfd()
{
  return IRfd;
}

/*------------------------------------------------------------*/

void
wbyte(c)
     u_char	c;
{
  dprintf((stderr, "> %02x\n", c));
  if (writetty(IRfd, &c, 1) < 0) {
    perror("writetty");
    Exit(1);
  }
}

u_char
rbyte()
{
  u_char	c;

  if (readtty(IRfd, &c, 1) < 0) {
    perror("readtty");
    Exit(1);
  }
  dprintf((stderr, "< %02x\n", c));
  return c;
}

inline void
wstr(p, len)
     u_char	*p;
     int	len;
{
  dprintf((stderr, "> len=%d\n", len));
  if (writetty(IRfd, p, len) < 0) {
    perror("writetty");
    Exit(1);
  }
}

inline void
rstr(p, len)
     u_char	*p;
     int	len;
{

  dprintf((stderr, "< len=%d\n", len));
  if (readtty(IRfd, p, len) < 0) {
    perror("readtty");
    Exit(1);
  }
}


/*       */
int
IRcheck()
{
  u_char u, v;
  setRTS(IRfd, 0);
  wbyte(0x40);
  u = rbyte();
  v = rbyte();
  setRTS(IRfd, 1);
  if((u == 'O') && (v == 'K'))
    return(1);
  return(0);
}

void
IRchangespeed(speed)
int speed;
{
  u_char u;
  switch(speed){
  case B38400:
    u = 0x72;
    break;
  case B19200:
    u = 0x52;
    break;
  case B9600:
  default:
    u = 0x32;
    break;
  }
  setRTS(IRfd, 0);
  wbyte(u);
  setRTS(IRfd, 1);
  changespeed(IRfd, speed);
}


void
IRreset()
{
  setRTS(IRfd, 0);
  wbyte(0x01);
  setRTS(IRfd, 1);
}

void
IRpresetRemocomMode()
{
  u_char u, v;
  setRTS(IRfd, 0);
  wbyte(0x31);
  setRTS(IRfd, 1);
}

int
IRsendPresetCommand(maker, command)
u_char maker;
u_char command;
{
  u_char u;
  wbyte(maker);
  wbyte(command);
  u = rbyte();
/*
  {
    int i;
    int j;
    for(i = 0 ; i < 10000 ; i++){
      j = getlinemode(IRfd);
      if(j != 0x162)
	printf(" %d>%04x \n", i, j);
    }
  }
*/
  return(u);
}

void
IRrecordRemocomMode()
{
  u_char u, v;
  setRTS(IRfd, 0);
  wbyte(0x41);
  setRTS(IRfd, 1);
}

int
IRrecordRemocomData(buf, buflen)
     u_char *buf;
     int buflen;
{
  u_char u;
  int len;
  len = 0;

  if(verbose)
    fprintf(stderr, "recording START.\n", len);

  while(len < buflen){
    u = rbyte();
    buf[len++] = u;
    if(len < 3) continue;
    if((u == 0x00) && (buf[len -3] == 0x00)){
      if(buf[len -2] == 0x40) {
/*	int i;
	fprintf(stderr,"len = %d\n", len);
	for (i = 0 ; i < len ; i++){
	  fprintf(stderr,"%02x ", buf[i]);
	}
	fprintf(stderr,"\n", len);
*/
/*	return(len -3); */
	if(verbose)
	  fprintf(stderr, "recording OK. %d bytes.\n", len);
	return(len);
      }
      if((buf[len -2] == 0xFF) || (buf[len -2] == 0xEE)){
	fprintf(stderr, "recording fail.\n");
	return(0);
      }
    }
  }
  return(len);
}

void
IRsendRemocomMode()
{
  u_char u, v;
  setRTS(IRfd, 0);
  wbyte(0x51);
  setRTS(IRfd, 1);
}

int
IRsendRemocomData(buf, buflen)
     u_char *buf;
     int buflen;
{
  int i;
  u_char u;
  for(i = 0 ; i < buflen ; i++){
    wbyte(buf[i]);
  }
  setRTS(IRfd, 1);
  u = rbyte();
}

void
IRversionInfo()
{
  u_char buf[32];
  setRTS(IRfd, 0);
  wbyte(0x20);
  setRTS(IRfd, 1);
  rstr(buf, 20);
  buf[20] = '\0';
  fprintf(stderr, "%s\n", buf);
  setRTS(IRfd, 0);
  wbyte(0x30);
  setRTS(IRfd, 1);
  rstr(buf, 13);
  buf[13] = '\0';
  fprintf(stderr, "%s\n", buf);
}
